==> qieyun-examples-20220801.0.0.tgz: package/mid_tang.js <==
/* 推導盛中唐擬音
 *
 * 中古漢語音變匯總：https://zhuanlan.zhihu.com/p/403479390
 * 本擬音是根據南北朝以來的音變，從《切韻》音系推出的。盛中唐期間的一些音變，以及擬音和記音方面的多種可能，均以選項形式列出
 * 
 * 預置風格說明：
 * - 盛唐(保守)：區分BC韻、清青、咍泰；章組歸在四等，區分常船；濁上獨立
 * - 中唐 慧琳 ：不分BC韻、清青、咍泰；章組歸在四等，不分常船；濁上歸去
 * - 中唐 韻圖 ：不分BC韻、清青、咍泰；莊章合爲照組，區分常船
 *
 * 擬音依據包括：
 * - 韻圖
 * - 玄應、慧琳音義和各家反切
 * - 詩歌碑誌銘文用韻
 * - 日、韓、越漢字音
 * - 梵語、藏語、突厥語、粟特語、回鶻語譯音
 * - 關於語音不正的記載和笑話
 *
 * @author unt
 */

const 非組字典 = {
  'pf': { 幫: 'pf', 滂: 'pfʰ', 並: 'bv', 明: 'mʋ' },
  'f': { 幫: 'f', 滂: 'fʰ', 並: 'v', 明: 'ɱ' }, // 微母脫鼻比非母脫塞晚很多
};
const 常船母字典 = {
  '常dʑ 船ʑ': [['常母', 'dʑ'], ['船母', 'ʑ']], // 注意如果分常船，則俟母也獨立
  '全ʑ': [['常船母', 'ʑ']],
  '全dʑ': [['常船母', 'dʑ']],
  '平dʑ 仄ʑ': [['常船母', [['平聲', 'dʑ'], ['', 'ʑ']]]],
};

if (!音韻地位) return [
  ['預置風格:', [3,
    '無',
    '盛唐（保守）',
    '中唐·慧琳反切',
    '中唐·韻圖',
  ]],
  ['非組:', [1].concat(Object.keys(非組字典))],
  ['二等介音:', [1, '省略', 'ʕ']], // 如需其他寫法請自行修改
  ['部分蟹攝二等入假攝', true],
  ['部分流攝脣音入遇攝', true],
  // ['遇攝用半高元音', true], // 模韻介於 u、o 之間，默認寫 u
  ['入聲韻尾:', [2, 'p t k', 'p ɾ k', 'β ɾ ɣ']], // 在盛中唐已開始弱化
  ['聲調:', [1, '五度符號', '附加符號', '調類數字']],
  ['　【以下選項會被預置風格覆蓋】莊章組:', [1,
    '莊tʂ二等 章tɕ四等',
    '莊tɕ二等 章tɕ三等', // 韻圖風格
  ]],
  ['常船母:', [2].concat(Object.keys(常船母字典))],
  // TODO: 鼻音塞化（暫時不實現）
  ['BC韻合流', true],
  ['清青合併', true],
  ['咍泰合併', true], // 盛唐可能還沒合併
  ['全濁上歸去', true],
];

for (const key in 選項) {
  選項[key.replace(/:/g, '')] = 選項[key]; // 去除冒號，方便下面代碼中引用
}
選項.莊章組 = 選項['　【以下選項會被預置風格覆蓋】莊章組'];
if (選項.預置風格 !== '無') {
  const is中唐 = 選項.預置風格.includes('中唐');
  const is韻圖 = 選項.預置風格.includes('韻圖');

  選項.BC韻合流 = is中唐;
  選項.清青合併 = is中唐;
  選項.咍泰合併 = is中唐;
  選項.莊章組 = is韻圖 ? '莊tɕ二等 章tɕ三等' : '莊tʂ二等 章tɕ四等';
  if (!is中唐 || is韻圖) {
    選項.常船母 = Object.keys(常船母字典)[0];
  } else if (選項.常船母 === Object.keys(常船母字典)[0]) {
    選項.常船母 = Object.keys(常船母字典)[1];
  }
  if (!is中唐) 選項.全濁上歸去 = false;
  if (is中唐 && !is韻圖) 選項.全濁上歸去 = true;
}

const is = (x) => 音韻地位.屬於(x);
const when = (...x) => 音韻地位.判斷(...x);
const 輕唇化條件 = '幫組 東鍾微虞廢文元歌陽尤凡韻 三等 非 重紐A類';

function 調整音韻地位() {
  function 調整(表達式, 調整屬性) { if (is(表達式)) 音韻地位 = 音韻地位.調整(調整屬性); }
  // 輕唇化例外
  調整('明母 尤韻', { 等: '一', 韻: '侯' });
  調整('明母 東韻', { 等: '一' });

  // [慧琳反切體現的, 唐代用韻體現的, 據今音推測的]
  const 蟹攝二等入假攝字 = ['崖咼(呙)扠涯搋派差絓畫(画)罣罷(罢)', '佳鼃娃解釵(钗)卦柴', '哇洼蛙灑蝸話(话)掛挂查叉杈衩'].join('');
  const 流攝脣音入遇攝字 = ['浮戊母罦罘蜉矛茂覆懋拇某負(负)阜', '謀(谋)部畝(亩)畮婦(妇)不否桴富牟缶', '復複(复)副牡'].join('');
  if (選項.部分蟹攝二等入假攝 && 蟹攝二等入假攝字.includes(字頭)) 調整('蟹攝 二等', { 韻: '麻' });
  if (選項.部分流攝脣音入遇攝 && 流攝脣音入遇攝字.includes(字頭)) 調整('幫組 尤侯韻', { 韻: is`尤韻` ? '虞' : '模' });

  if (選項.常船母 !== Object.keys(常船母字典)[0]) 調整('俟母', { 母: '船' });

  if (選項.全濁上歸去) 調整('全濁 上聲', { 聲: '去' });
}

function get聲母() {
  let 聲母 = when([
    [輕唇化條件, 非組字典[選項.非組][音韻地位.母]],
    ['常船母', 常船母字典[選項.常船母]],
    ['', {
      // 儘管全濁聲母在盛中唐在音值上已經弛化乃至清化，但是仍可使用濁音符號
      // 鼻音塞化也不需要寫出來，而且它只是長安音的特色
      幫: 'p', 滂: 'pʰ', 並: 'b', 明: 'm',
      端: 't', 透: 'tʰ', 定: 'd', 泥: 'n', 來: 'l',
      知: 'ʈ', 徹: 'ʈʰ', 澄: 'ɖ', 孃: 'ɳ',
      精: 'ts', 清: 'tsʰ', 從: 'dz', 心: 's', 邪: 'z',
      莊: 'tʂ', 初: 'tʂʰ', 崇: 'dʐ', 生: 'ʂ', 俟: 'ʐ',
      章: 'tɕ', 昌: 'tɕʰ', 常: 'dʑ', 書: 'ɕ', 船: 'ʑ', 日: 'ɲ', 以: 'j',
      見: 'k', 溪: 'kʰ', 羣: 'ɡ', 疑: 'ŋ',
      影: 'ʔ', 曉: 'x', 匣: 'ɣ', 云: '',
    }[音韻地位.母]],
  ]);
  if (!選項.莊章組.includes('ʂ')) {
    聲母 = 聲母.replace('ʂ', 'ɕ').replace('ʐ', 'ʑ');
    聲母 = 聲母.replace('j', ''); // 以云也合爲喻母
  }
  return 聲母;
}

function get韻母() {
  let 韻圖等 = when([
    ['精組 止攝 開口', '一'],
    ['莊組', '二'],
    ['章組 或 日母', 選項.莊章組.includes('四') ? '四' : '三'],
    ['非 三等', 音韻地位.等], // 切韻一二四等到韻圖不變

    [輕唇化條件, '輕'], // 輕唇算作一個獨立的等
    ['以母 或 重紐A類 或 鈍音 麻清韻 或 見影組 幽韻', '四'], // 鈍音切韻三 A
    ['端精組', '四'],
    ['', '三'] // 其他切韻三等到韻圖不變
  ]);

  let 韻母列表 = when([
    // 分開合的韻，用豎線分隔開口韻母和合口韻母，之後篩選
    // 輕唇韻母同下一行的韻母去掉介音
    ['遇攝', {
      輕: 'u',
      一: 'u',
      二: 'ʕɯ|ʕu',
      三: 'ɨɯ|ʉu',
      四: 'iɯ|yu'
    }],
    ['果假攝', {
      輕: 'uɒ',
      一: 'ɑ|uɒ',
      二: 'ʕa|ʕua',
      三: 'ɨɑ|ʉɒ',
      四: 'ia|ya'
    }],

    ['曾攝', {
      一: 'əŋ|uəŋ',
      二: 'ʕəŋ|ʕuəŋ',
      三: 'ɨŋ|ʉɨŋ',
      四: 'iɨŋ|yɨŋ',
    }],
    ['梗攝', {
      二: 'ʕɛŋ|ʕuɛŋ',
      三: 'ɨɛŋ|ʉɛŋ', // 清青合併前，庚三和庚二是相同的元音
      四: 選項.清青合併 || !is`青韻` ? 'iɛŋ|yɛŋ' : 'ieŋ|yeŋ',
    }],

    ['通攝', {
      一: 'oŋ',
      二: 'ʕoŋ',
      輕: is`東韻` ? 'uŋ' : 'oŋ',
      三: is`東韻` ? 'ɨuŋ' : 'ʉoŋ',
      四: is`東韻` ? 'iuŋ' : 'yoŋ',
    }],
    ['江攝', {
      二: 'ʕœŋ',
    }],
    ['宕攝', {
      輕: 'ɑŋ',
      一: 'ɑŋ|uɑŋ',
      二: 'ʕɑŋ|ʕuɑŋ',
      三: 'ɨɑŋ|ʉɑŋ',
      四: 'iɑŋ|yɑŋ',
    }],

    ['止攝', {
      一: 'ɹ̩',
      二: 'ʕɹ̩|ʕuɹ̩',
      輕: 選項.BC韻合流 ? 'ʉi' : 'ʉj',
      三: 選項.BC韻合流 || !is`微韻` ? 'ɨi|ʉi' : 'ɨj|ʉj',
      四: 'i|yi',
    }],
    ['蟹攝', {
      一: 選項.咍泰合併 ? 'ɑj|uɔj' : is`泰韻` ? 'ɑj|uɑj' : 'ʌj|uɔj',
      二: 'ʕaj|ʕuaj',
      三: 選項.BC韻合流 && is`見影組` || is`廢韻` ? 'ɨɜj|ʉɜj' : 'ɨɛj|ʉɛj',
      輕: 'iɛj',
      四: 'iɛj|yɛj',
    }],

    ['臻攝 非 元韻', {
      一: 'ən|un',
      二: 'ʕən|ʕun',
      輕: 'ʉn',
      三: 選項.BC韻合流 && is`見影組` || is`欣文韻` ? 'ɨn|ʉn' : 'ɨin|ʉin',
      四: 'in|yn',
    }],
    ['山攝 或 元韻', {
      一: 'ɑn|uɒn',
      二: 'ʕan|ʕuan',
      輕: 'ɜn',
      三: 選項.BC韻合流 && is`見影組` || is`元韻` ? 'ɨɜn|ʉɜn' : 'ɨɛn|ʉɛn',
      四: 'iɛn|yɛn',
    }],

    ['流攝', {
      輕: 'əw',
      一: 'əw',
      二: 'ʕəw',
      三: 'ɨw',
      四: 'iw',
    }],
    ['效攝', {
      一: 'ʌw',
      二: 'ʕaw',
      三: 'ɨɛw',
      四: 'iɛw',
    }],

    ['深攝', {
      二: 'ʕəm',
      三: 選項.BC韻合流 && is`見影組` ? 'ɨm' : 'ɨim',
      四: 'im',
    }],
    ['咸攝', {
      一: 'ɑm',
      二: 'ʕam',
      輕: 'ɜm',
      三: 選項.BC韻合流 && is`見影組` || is`嚴凡韻` ? 'ɨɜm' : 'ɨɛm',
      四: 'iɛm',
    }],
  ])[韻圖等].split('|');

  let 開合index = when([
    [韻母列表.length === 1, 0],
    ['虞韻 或 幫組 一等 非 曾宕攝', 1],
    ['開口 或 幫組', 0],
    ['合口', 1],
  ]);

  let 韻母 = 韻母列表[開合index];
  if (is`入聲`) {
    ['m', 'n', 'ŋ'].some((e, i) => {
      if (韻母.slice(-1) === e) {
        韻母 = 韻母.slice(0, -1) + 選項.入聲韻尾.split(' ')[i];
        return true;
      }
    });
  }

  韻母 = 韻母.replace('ʕ', 選項.二等介音.replace('省略', ''));

  if (選項.遇攝用半高元音 && ['ɯ', 'u'].includes(韻母.slice(-1))) {
    韻母 = 韻母.replace('ɯ', 'ɤ').replace('u', 'o');
  }
  return 韻母;
}

function get聲調() {
  return {
    五度符號: {
      平: is`全清 或 次清` ? '˦˨' : '˨˩',
      上: !is`全濁` ? '˦˥' : '˨˦',
      去: is`全清 或 次清` ? '˧˨˦' : '˨˨˦',
      入: is`全清 或 次清` ? '˦' : '˨',
    },
    附加符號: { 平: '̀', 上: '́', 去: '̌', 入: '' },
    調類數字: { 平: '¹', 上: '²', 去: '³', 入: '⁴' },
  }[選項.聲調][音韻地位.聲];
}

function get音節() {
  const 音節 = {
    聲母: get聲母(),
    韻母: get韻母(),
    聲調: get聲調(),
  };
  const 韻尾列表 = ['ŋ', 'j', 'n', 'w', 'm'] + 選項.入聲韻尾.split(' ');
  音節.帶調韻母 = 選項.聲調 === '附加符號' && 韻尾列表.includes(音節.韻母.slice(-1)) ?
    音節.韻母.slice(0, -1) + 音節.聲調 + 音節.韻母.slice(-1) :
    音節.韻母 + 音節.聲調;
  return 音節;
}

調整音韻地位();
const 音節 = get音節();
return 音節.聲母 + 音節.帶調韻母;

==> qieyun-examples-20220826.0.1.tgz: package/mid_tang.js <==
/* 推導盛中唐擬音
 *
 * 中古漢語音變匯總：https://zhuanlan.zhihu.com/p/403479390
 * 本擬音是根據南北朝以來的音變，從《切韻》音系推出的。盛中唐期間的一些音變，以及擬音和記音方面的多種可能，均以選項形式列出
 *
 * 預置風格說明：
 * - 盛唐(保守)：區分BC韻、清青、咍泰；章組歸在四等，區分常船；濁上獨立
 * - 中唐 慧琳 ：不分BC韻、清青、咍泰；章組歸在四等，不分常船；濁上歸去
 * - 中唐 韻圖 ：不分BC韻、清青、咍泰；莊章合爲照組，區分常船
 *
 * 擬音依據包括：
 * - 韻圖
 * - 玄應、慧琳音義和各家反切
 * - 詩歌碑誌銘文用韻
 * - 日、韓、越漢字音
 * - 梵語、藏語、突厥語、粟特語、回鶻語譯音
 * - 關於語音不正的記載和笑話
 *
 * @author unt
 */

const is = (...x) => 音韻地位.屬於(...x);
const when = (...x) => 音韻地位.判斷(...x);

const has預置風格 = !選項.預置風格 || 選項.預置風格 !== '無';
const is盛唐 = 選項.預置風格?.includes('盛唐') ?? false;
const is中唐 = 選項.預置風格?.includes('中唐') ?? true;
const is慧琳 = 選項.預置風格?.includes('慧琳') ?? true;
const is韻圖 = 選項.預置風格?.includes('韻圖') ?? false;
const 輕唇化條件 = '幫組 東鍾微虞廢文元歌陽尤凡韻 三等 非 重紐A類';

const 非組字典 = {
  'pf': { 幫: 'pf', 滂: 'pfʰ', 並: 'bv', 明: 'mʋ' },
  'f': { 幫: 'f', 滂: 'fʰ', 並: 'v', 明: 'ɱ' }, // 微母脫鼻比非母脫塞晚很多
};
const 常船母字典 = {
  '常dʑ 船ʑ': [],
  '全ʑ': [['常船母', 'ʑ'], ['俟母', 'dʐ']], // 如果不分常船，則俟母也不獨立
  '全dʑ': [['常船母', 'dʑ'], ['俟母', 'dʐ']],
  '平dʑ 仄ʑ': [['常船母', [['平聲', 'dʑ'], ['', 'ʑ']]], ['俟母', 'dʐ']],
};

if (!音韻地位) return [
  ['預置風格', [3,
    '無',
    '盛唐（保守）',
    '中唐·慧琳反切',
    '中唐·韻圖',
  ]],

  '聲',
  ['非組', [1].concat(Object.keys(非組字典))],
  ['莊章組', has預置風格 ? null : [1,
    '莊tʂ二等 章tɕ四等',
    '莊tɕ二等 章tɕ三等', // 韻圖風格
  ]],
  ['常船母', [1].concat(Object.keys(常船母字典).slice(is慧琳 ? 1 : 0))
    .map(e => is慧琳 ? e + ' ' : e) // 加空格是爲了在切換預置風格時刷新選中的內容
  ],
  // TODO: 鼻音塞化（暫時不實現）

  '韻',
  ['二等介音', [1, '省略', 'ʕ']], // 如需其他寫法請自行修改
  ['入聲韻尾', [1, 'p t k', 'p ɾ k', 'β ɾ ɣ']],
  ['部分蟹攝二等入假攝', true],
  ['部分流攝脣音入遇攝', true],
  // ['遇攝用半高元音', true], // 模韻介於 u、o 之間，默認寫 u
  ['BC韻合流', has預置風格 ? null : true],
  ['清青合併', has預置風格 ? null : true],
  ['咍泰合併', has預置風格 ? null : true], // 盛唐可能還沒合併

  '調',
  ['聲調', [1, '五度符號', '附加符號', '調類數字']],
  ['全濁上歸去', has預置風格 && !is韻圖 ? null : true],
  '', // 在 v0.1 推導器中使「恢復預設值」按鈕位於下一行
];

選項.常船母 = 選項.常船母.trim();
if (has預置風格) {
  選項.BC韻合流 = is中唐;
  選項.清青合併 = is中唐;
  選項.咍泰合併 = is中唐;
  選項.莊章組 = is韻圖 ? '莊tɕ二等 章tɕ三等' : '莊tʂ二等 章tɕ四等';
  if (!is韻圖) 選項.全濁上歸去 = is中唐;
}

function 調整音韻地位() {
  function 調整(表達式, 調整屬性) { if (is(表達式)) 音韻地位 = 音韻地位.調整(調整屬性); }
  // 輕唇化例外
  調整('明母 尤韻', { 等: '一', 韻: '侯' });
  調整('明母 東韻', { 等: '一' });

  // [慧琳反切體現的, 唐代用韻體現的, 據今音推測的]
  const 蟹攝二等入假攝字 = ['崖咼(呙)扠涯搋派差絓畫(画)罣罷(罢)', '佳鼃娃解釵(钗)卦柴', '哇洼蛙灑蝸話(话)掛挂查叉杈衩'].join('');
  const 流攝脣音入遇攝字 = ['浮戊母罦罘蜉矛茂覆懋拇某負(负)阜', '謀(谋)部畝(亩)畮婦(妇)不否桴富牟缶', '復複(复)副牡'].join('');
  if (選項.部分蟹攝二等入假攝 && 蟹攝二等入假攝字.includes(字頭)) 調整('蟹攝 二等', { 韻: '麻' });
  if (選項.部分流攝脣音入遇攝 && 流攝脣音入遇攝字.includes(字頭)) 調整('幫組 尤侯韻', { 韻: is`尤韻` ? '虞' : '模' });

  if (選項.全濁上歸去) 調整('全濁 上聲', { 聲: '去' });
}

function get聲母() {
  let 聲母 = when([
    [輕唇化條件, 非組字典[選項.非組][音韻地位.母]],
    ...常船母字典[選項.常船母],
    ['', {
      // 儘管全濁聲母在盛中唐在音值上已經弛化乃至清化，但是仍可使用濁音符號
      // 鼻音塞化也不需要寫出來，而且它只是長安音的特色
      幫: 'p', 滂: 'pʰ', 並: 'b', 明: 'm',
      端: 't', 透: 'tʰ', 定: 'd', 泥: 'n', 來: 'l',
      知: 'ʈ', 徹: 'ʈʰ', 澄: 'ɖ', 孃: 'ɳ',
      精: 'ts', 清: 'tsʰ', 從: 'dz', 心: 's', 邪: 'z',
      莊: 'tʂ', 初: 'tʂʰ', 崇: 'dʐ', 生: 'ʂ', 俟: 'ʐ',
      章: 'tɕ', 昌: 'tɕʰ', 常: 'dʑ', 書: 'ɕ', 船: 'ʑ', 日: 'ɲ', 以: 'j',
      見: 'k', 溪: 'kʰ', 羣: 'ɡ', 疑: 'ŋ',
      影: 'ʔ', 曉: 'x', 匣: 'ɣ', 云: '',
    }[音韻地位.母]],
  ]);
  if (!選項.莊章組.includes('ʂ')) {
    聲母 = 聲母.replace('ʂ', 'ɕ').replace('ʐ', 'ʑ');
    聲母 = 聲母.replace('j', ''); // 以云也合爲喻母
  }
  return 聲母;
}

function get韻母() {
  let 韻圖等 = when([
    ['精組 止攝 開口', '一'],
    ['莊組', '二'],
    ['章組 或 日母', 選項.莊章組.includes('四') ? '四' : '三'],
    ['非 三等', 音韻地位.等], // 切韻一二四等到韻圖不變

    [輕唇化條件, '輕'], // 輕唇算作一個獨立的等
    ['以母 或 重紐A類 或 鈍音 麻清韻 或 見影組 幽韻', '四'], // 鈍音切韻三 A
    ['端精組', '四'],
    ['', '三'] // 其他切韻三等到韻圖不變
  ]);

  let 韻母列表 = when([
    // 分開合的韻，用豎線分隔開口韻母和合口韻母，之後篩選
    // 輕唇韻母同下一行的韻母去掉介音
    ['遇攝', {
      輕: 'u',
      一: 'u',
      二: 'ʕɯ|ʕu',
      三: 'ɨɯ|ʉu',
      四: 'iɯ|yu'
    }],
    ['果假攝', {
      輕: 'uɒ',
      一: 'ɑ|uɒ',
      二: 'ʕa|ʕua',
      三: 'ɨɑ|ʉɒ',
      四: 'ia|ya'
    }],

    ['曾攝', {
      一: 'əŋ|uəŋ',
      二: 'ʕəŋ|ʕuəŋ',
      三: 'ɨŋ|ʉɨŋ',
      四: 'iɨŋ|yɨŋ',
    }],
    ['梗攝', {
      二: 'ʕɛŋ|ʕuɛŋ',
      三: 'ɨɛŋ|ʉɛŋ', // 清青合併前，庚三和庚二是相同的元音
      四: 選項.清青合併 || !is`青韻` ? 'iɛŋ|yɛŋ' : 'ieŋ|yeŋ',
    }],

    ['通攝', {
      一: 'oŋ',
      二: 'ʕoŋ',
      輕: is`東韻` ? 'uŋ' : 'oŋ',
      三: is`東韻` ? 'ɨuŋ' : 'ʉoŋ',
      四: is`東韻` ? 'iuŋ' : 'yoŋ',
    }],
    ['江攝', {
      二: 'ʕœŋ',
    }],
    ['宕攝', {
      輕: 'ɑŋ',
      一: 'ɑŋ|uɑŋ',
      二: 'ʕɑŋ|ʕuɑŋ',
      三: 'ɨɑŋ|ʉɑŋ',
      四: 'iɑŋ|yɑŋ',
    }],

    ['止攝', {
      一: 'ɹ̩',
      二: 'ʕɹ̩|ʕuɹ̩',
      輕: 選項.BC韻合流 ? 'ʉi' : 'ʉj',
      三: 選項.BC韻合流 || !is`微韻` ? 'ɨi|ʉi' : 'ɨj|ʉj',
      四: 'i|yi',
    }],
    ['蟹攝', {
      一: 選項.咍泰合併 ? 'ɑj|uɔj' : is`泰韻` ? 'ɑj|uɑj' : 'ʌj|uɔj',
      二: 'ʕaj|ʕuaj',
      三: 選項.BC韻合流 && is`見影組` || is`廢韻` ? 'ɨɜj|ʉɜj' : 'ɨɛj|ʉɛj',
      輕: 'iɛj',
      四: 'iɛj|yɛj',
    }],

    ['臻攝 非 元韻', {
      一: 'ən|un',
      二: 'ʕən|ʕun',
      輕: 'ʉn',
      三: 選項.BC韻合流 && is`見影組` || is`欣文韻` ? 'ɨn|ʉn' : 'ɨin|ʉin',
      四: 'in|yn',
    }],
    ['山攝 或 元韻', {
      一: 'ɑn|uɒn',
      二: 'ʕan|ʕuan',
      輕: 'ɜn',
      三: 選項.BC韻合流 && is`見影組` || is`元韻` ? 'ɨɜn|ʉɜn' : 'ɨɛn|ʉɛn',
      四: 'iɛn|yɛn',
    }],

    ['流攝', {
      輕: 'əw',
      一: 'əw',
      二: 'ʕəw',
      三: 'ɨw',
      四: 'iw',
    }],
    ['效攝', {
      一: 'ʌw',
      二: 'ʕaw',
      三: 'ɨɛw',
      四: 'iɛw',
    }],

    ['深攝', {
      二: 'ʕəm',
      三: 選項.BC韻合流 && is`見影組` ? 'ɨm' : 'ɨim',
      四: 'im',
    }],
    ['咸攝', {
      一: 'ɑm',
      二: 'ʕam',
      輕: 'ɜm',
      三: 選項.BC韻合流 && is`見影組` || is`嚴凡韻` ? 'ɨɜm' : 'ɨɛm',
      四: 'iɛm',
    }],
  ])[韻圖等].split('|');

  let 開合index = when([
    [韻母列表.length === 1, 0],
    ['虞韻 或 幫組 一等 非 曾宕攝', 1],
    ['開口 或 幫組', 0],
    ['合口', 1],
  ]);

  let 韻母 = 韻母列表[開合index];
  if (is`入聲`) {
    選項.入聲韻尾.split(' ').forEach((e, i) => { 韻母 = 韻母.replace('mnŋ'[i], e); });
  }

  韻母 = 韻母.replace('ʕ', 選項.二等介音.replace('省略', ''));

  if (選項.遇攝用半高元音 && ['ɯ', 'u'].includes(韻母.slice(-1))) {
    韻母 = 韻母.replace('ɯ', 'ɤ').replace('u', 'o');
  }
  return 韻母;
}

function get聲調() {
  const is陰 = is`全清 或 次清 或 次濁 上聲`;
  return {
    五度符號: is陰 ?
      { 平: '˦˨', 上: '˦˥', 去: '˧˨˦', 入: '˦' } :
      { 平: '˨˩', 上: '˨˦', 去: '˨˨˦', 入: '˨' },
    附加符號: { 平: '̀', 上: '́', 去: '̌', 入: '' },
    調類數字: { 平: '¹', 上: '²', 去: '³', 入: '⁴' },
  }[選項.聲調][音韻地位.聲];
}

function get音節() {
  const 音節 = {
    聲母: get聲母(),
    韻母: get韻母(),
    聲調: get聲調(),
  };
  const 韻尾列表 = [...'ŋjnwm', ...選項.入聲韻尾.split(' ')];
  音節.帶調韻母 = 選項.聲調 === '附加符號' && 韻尾列表.includes(音節.韻母.slice(-1)) ?
    音節.韻母.slice(0, -1) + 音節.聲調 + 音節.韻母.slice(-1) :
    音節.韻母 + 音節.聲調;
  return 音節;
}

調整音韻地位();
const 音節 = get音節();
return 音節.聲母 + 音節.帶調韻母;
